"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require("babel-runtime/core-js/get-iterator");

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _elasticlunr = require("elasticlunr");

var _elasticlunr2 = _interopRequireDefault(_elasticlunr);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  the mixin class  */
class gtsUtilFTS {
    /*  mixin initialization  */
    initializer(sequelize, options) {
        this._ftsCfg = typeof options.fts === "object" ? options.fts : null;
        this._ftsIdx = {};
    }

    /*  cherry-pick fields for FTS indexing  */
    _ftsObj2Doc(type, obj) {
        let id = String(obj.id);
        let doc = { id: id, __any: id };
        this._ftsCfg[type].forEach(field => {
            let val = String(obj[field]);
            doc[field] = val;
            doc.__any += ` ${val}`;
        });
        return doc;
    }

    /*  bootstrap FTS by creating initial in-memory index  */
    _ftsBoot() {
        var _this = this;

        return (0, _asyncToGenerator3.default)(function* () {
            /*  operate only if FTS is configured  */
            if (_this._ftsCfg === null) return;

            /*  iterate over all entity types...  */
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = (0, _getIterator3.default)((0, _keys2.default)(_this._ftsCfg)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    let type = _step.value;

                    /*  create a new in-memory index  */
                    _this._ftsIdx[type] = new _elasticlunr2.default.Index();
                    _this._ftsIdx[type].saveDocument(false);
                    _this._ftsIdx[type].addField("id");
                    _this._ftsIdx[type].addField("__any");
                    _this._ftsCfg[type].forEach(function (field) {
                        _this._ftsIdx[type].addField(field);
                    });
                    _this._ftsIdx[type].setRef("id");

                    /*  iterate over all entity objects...  */
                    let opts = { attributes: _this._ftsCfg[type].concat(["id"]) };
                    let objs = yield _this._models[type].findAll(opts);
                    objs.forEach(function (obj) {
                        /*  add entity objects to index  */
                        let doc = _this._ftsObj2Doc(type, obj);
                        _this._ftsIdx[type].addDoc(doc);
                    });
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        })();
    }

    /*  update the FTS index  */
    _ftsUpdate(type, oid, obj, op) {
        /*  operate only if FTS is configured  */
        if (this._ftsCfg === null) return;
        if (this._ftsCfg[type] === undefined) return;

        /*  dispatch according to operation  */
        if (op === "create") {
            /*  add entity to index  */
            let doc = this._ftsObj2Doc(type, obj);
            this._ftsIdx[type].addDoc(doc);
        } else if (op === "update") {
            /*  update entity in index  */
            let doc = this._ftsObj2Doc(type, obj);
            this._ftsIdx[type].updateDoc(doc);
        } else if (op === "delete") {
            /*  delete entity from index  */
            this._ftsIdx[type].removeDocByRef(oid);
        }
    }

    /*  search in the FTS index  */
    _ftsSearch(type, query, order, offset, limit, ctx) {
        /*  operate only if FTS is configured  */
        if (this._ftsCfg === null) return new Error("Full-Text-Search (FTS) not available at all");
        if (this._ftsCfg[type] === undefined) return new Error(`Full-Text-Search (FTS) not available for entity "${type}"`);

        /*  parse "[field:]keyword [field:]keyword [, ...]" query string  */
        let queries = [];
        query.split(/\s*,\s*/).forEach(query => {
            let fields = {};
            query.split(/\s+/).forEach(field => {
                let fn = "__any";
                let kw = field;
                let m;
                if ((m = field.match(/^(.+):(.+)$/)) !== null) {
                    fn = m[1];
                    kw = m[2];
                }
                if (fn !== "__any" && this._ftsCfg[type].indexOf(fn) < 0) throw new Error(`Full-Text-Search (FTS) not available for field "${fn}" of entity "${type}"`);
                if (fields[fn] === undefined) fields[fn] = [];
                fields[fn].push(kw);
            });
            queries.push(fields);
        });

        /*  iterate over all queries...  */
        let results1 = {};
        queries.forEach(query => {
            /*   iterate over all fields...  */
            let results2 = {};
            (0, _keys2.default)(query).forEach(field => {
                /*  lookup entity ids from index for particular field  */
                let kw = query[field].join(" ");
                let config = {
                    fields: {
                        [field]: {
                            boost: 1,
                            expand: true,
                            bool: "AND"
                        }
                    }
                };
                let results = this._ftsIdx[type].search(kw, config);

                /*  reduce result list to set of unique ids  */
                let results3 = {};
                results.forEach(result => {
                    let oid = result.ref;
                    results3[oid] = true;
                });

                /*  AND-combine results with previous results  */
                let oids = (0, _keys2.default)(results2);
                if (oids.length === 0) (0, _keys2.default)(results3).forEach(oid => {
                    results2[oid] = true;
                });else {
                    oids.forEach(oid => {
                        if (!results3[oid]) delete results2[oid];
                    });
                }
            });

            /*  OR-combine results with previous results  */
            (0, _keys2.default)(results2).forEach(oid => {
                results1[oid] = true;
            });
        });

        /*  query entity objects from database  */
        let opts = { where: { id: (0, _keys2.default)(results1) } };
        if (order !== undefined) opts.order = order;
        if (offset !== undefined) opts.offset = offset;
        if (limit !== undefined) opts.limit = limit;
        if (ctx.tx !== undefined) opts.transaction = ctx.tx;
        return this._models[type].findAll(opts);
    }
}
exports.default = gtsUtilFTS; /*
                              **  GraphQL-Tools-Sequelize -- Integration of GraphQL-Tools and Sequelize ORM
                              **  Copyright (c) 2016-2017 Ralf S. Engelschall <rse@engelschall.com>
                              **
                              **  Permission is hereby granted, free of charge, to any person obtaining
                              **  a copy of this software and associated documentation files (the
                              **  "Software"), to deal in the Software without restriction, including
                              **  without limitation the rights to use, copy, modify, merge, publish,
                              **  distribute, sublicense, and/or sell copies of the Software, and to
                              **  permit persons to whom the Software is furnished to do so, subject to
                              **  the following conditions:
                              **
                              **  The above copyright notice and this permission notice shall be included
                              **  in all copies or substantial portions of the Software.
                              **
                              **  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
                              **  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
                              **  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
                              **  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
                              **  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
                              **  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
                              **  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
                              */

/*  external dependencies  */
